#ifndef _LIBNETFILTER_XTABLES_XTNETLINK_H
#define _LIBNETFILTER_XTABLES_XTNETLINK_H 1

#ifdef __cplusplus
#	include <climits>
#else
#	include <limits.h>
#endif
#if defined(__cplusplus) && \
    (__cplusplus >= 201100L || defined(__GXX_EXPERIMENTAL_CXX0X__))
#	include <cstdint>
#else
#	include <stdint.h>
#endif
#include <linux/netfilter/nfnetlink_xtables.h> /* grab error codes */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Possible flags to xtnetlink_open().
 * %XTNL_NO_FLAGS:	mnemonic for none
 */
enum {
	XTNL_NO_FLAGS = 0,
};

enum {
	XTNL_CHAIN_APPEND = UINT_MAX,
	XTNL_CHAIN_FLUSH = UINT_MAX,
};

enum xtnetlink_ident_property {
	XTNETLINK_IDPROP_IMPNAME,
	XTNETLINK_IDPROP_REVMIN,
	XTNETLINK_IDPROP_REVMAX,
};

struct xtnetlink_promote_param {
	const char *name;
	unsigned int hooknum;
	int priority;
	uint8_t nfproto;
};

struct xtnetlink_handle;
struct xtnetlink_identify;
struct xtuser_chain;
struct xtuser_table;

/* Socket setup */
extern struct xtnetlink_handle *xtnetlink_open(unsigned int);
extern int xtnetlink_close(struct xtnetlink_handle *);

/* Chain operations */
extern int xtnetlink_chain_create(struct xtnetlink_handle *,
	const struct xtuser_chain *);
extern int xtnetlink_chain_del(struct xtnetlink_handle *,
	const char *);
extern int xtnetlink_chain_move(struct xtnetlink_handle *,
	const char *, const char *);
extern int xtnetlink_chain_splice(struct xtnetlink_handle *,
	const struct xtuser_chain *, unsigned int, unsigned int);

extern struct xtuser_chain *xtnetlink_chain_dump(struct xtnetlink_handle *,
	const char *);
extern int xtnetlink_chain_promote(struct xtnetlink_handle *,
	const struct xtnetlink_promote_param *);
extern int xtnetlink_chain_demote(struct xtnetlink_handle *, const char *);

/* Table operations */
extern int xtnetlink_table_replace(struct xtnetlink_handle *,
	const struct xtuser_table *);
extern struct xtuser_table *xtnetlink_table_dump(struct xtnetlink_handle *);

/* Auxiliary */
extern int xtnetlink_identify(struct xtnetlink_handle *,
	struct xtnetlink_identify **);
extern int xtnetlink_ident_propget(const struct xtnetlink_identify *,
	enum xtnetlink_ident_property, void *);
extern void xtnetlink_ident_free(struct xtnetlink_identify *);
extern int xtnetlink_test_error(struct xtnetlink_handle *);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _LIBNETFILTER_XTABLES_XTNETLINK_H */
