#ifndef _LIBNETFILTER_XTABLES_XTUSER_H
#define _LIBNETFILTER_XTABLES_XTUSER_H 1

#ifndef __cplusplus
#	include <stdbool.h>
#endif
#if defined(__cplusplus) && \
    (__cplusplus >= 201100L || defined(__GXX_EXPERIMENTAL_CXX0X__))
#	include <cstdint>
#else
#	include <stdint.h>
#endif
#include <sys/types.h>
#include <linux/netfilter/nfnetlink_xtables.h> /* grab error codes */

enum xtuser_action_type {
	XTUSER_ACTION_NONE = 0,
	XTUSER_ACTION_VERDICT,
	XTUSER_ACTION_MATCH,
	XTUSER_ACTION_TARGET,
	XTUSER_ACTION_JUMP,
	XTUSER_ACTION_GOTO,
};

enum xtuser_action_property {
	XTUSER_APROP_NONE,
	XTUSER_APROP_TYPE,
	XTUSER_APROP_VERDICT,
	XTUSER_APROP_NAME,
	XTUSER_APROP_REVISION,
	XTUSER_APROP_DATA,
	XTUSER_APROP_DATASIZE,
};

enum xtuser_chain_property {
	XTUSER_CPROP_NONE,
	XTUSER_CPROP_NAME,
	XTUSER_CPROP_HOOKNUM,
	XTUSER_CPROP_HOOKPRIO,
	XTUSER_CPROP_HOOKNFPROTO,
	XTUSER_CPROP_SIZE,
};

enum {
	/* If XTUSER_INVALID_HOOK is used, prio and nfproto have no meaning. */
	XTUSER_INVALID_HOOK = UINT8_MAX,

	/*
	 * Any value >= NFPROTO_NUMPROTO is invalid, but let's try doing
	 * without #include <linux/netfilter.h>.
	 */
	XTUSER_INVALID_NFPROTO = UINT8_MAX,

	/* libnetfilter_xtables-specific error codes */
	NFXTE_EMSGSIZE = 0x100001,
};

struct xtuser_action;
struct xtuser_rule;
struct xtuser_rule_trav;
struct xtuser_chain;
struct xtuser_chain_trav;
struct xtuser_table;
struct xtuser_table_trav;

typedef bool (*xtuser_chain_travfunc_t)(const struct xtuser_chain *,
	const struct xtuser_rule *, void *);
typedef bool (*xtuser_table_travfunc_t)(const struct xtuser_table *,
	const struct xtuser_chain *, void *);

#ifdef __cplusplus
extern "C" {
#endif

/* General */
extern int xtuser_init(void);
extern void xtuser_exit(void);
extern const char *xtuser_strerror(int);

/* Action */
extern struct xtuser_action *
	xtuser_action_by_idx(const struct xtuser_rule *, unsigned int);
extern int xtuser_action_propget(const struct xtuser_action *,
	enum xtuser_action_property, void *);

/* Rules */
extern struct xtuser_rule *xtuser_rule_new(void);
extern void xtuser_rule_free(struct xtuser_chain *, struct xtuser_rule *);

extern struct xtuser_rule *
	xtuser_rule_by_idx(const struct xtuser_chain *, unsigned int);
extern struct xtuser_rule_iter *
	xtuser_rule_travinit(const struct xtuser_rule *);
extern struct xtuser_action *xtuser_rule_traverse(struct xtuser_rule_iter *);
extern void xtuser_rule_travfree(struct xtuser_rule_iter *);

extern int xtuser_rule_push_verdict(struct xtuser_rule *, unsigned int);
extern int xtuser_rule_push_match(struct xtuser_rule *, const char *,
	unsigned int, const void *, size_t);
extern int xtuser_rule_push_target(struct xtuser_rule *, const char *,
	unsigned int, const void *, size_t);
extern int xtuser_rule_push_jump(struct xtuser_rule *, const char *);
extern int xtuser_rule_push_goto(struct xtuser_rule *, const char *);

/* Chains */
extern struct xtuser_chain *xtuser_chain_create(struct xtuser_table *,
	const char *);
extern struct xtuser_chain *xtuser_chain_unlink(struct xtuser_table *,
	const char *);
extern void xtuser_chain_free(struct xtuser_chain *);
extern void xtuser_chain_flush(struct xtuser_chain *);

extern void xtuser_chain_foreach(const struct xtuser_chain *,
	xtuser_chain_travfunc_t, void *);
extern struct xtuser_chain_iter *
	xtuser_chain_travinit(const struct xtuser_chain *);
extern struct xtuser_rule *xtuser_chain_traverse(struct xtuser_chain_iter *);
extern void xtuser_chain_travfree(struct xtuser_chain_iter *);
extern struct xtuser_chain *xtuser_chain_lookup(const struct xtuser_table *,
	const char *);

extern int xtuser_chain_propget(const struct xtuser_chain *,
	enum xtuser_chain_property, void *);
extern int xtuser_chain_propset(struct xtuser_chain *,
	enum xtuser_chain_property, ...);
extern int xtuser_chain_push(struct xtuser_chain *, struct xtuser_rule *);

/* The table */
extern struct xtuser_table *xtuser_table_new(void);
extern void xtuser_table_free(struct xtuser_table *);
extern void xtuser_table_foreach(const struct xtuser_table *,
	xtuser_table_travfunc_t, void *);
extern struct xtuser_table_iter *
	xtuser_table_travinit(const struct xtuser_table *);
extern struct xtuser_chain *xtuser_table_traverse(struct xtuser_table_iter *);
extern void xtuser_table_travfree(struct xtuser_table_iter *);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _LIBNETFILTER_XTABLES_XTUSER_H */
